@@ -7,6 +7,7 @@ import glob
 import psutil
 import warnings
 import weakref
+import shutil
 import io as sio
 from yggdrasil import tools, platform, serialize
 from yggdrasil.languages import get_language_dir
@@ -15,8 +16,10 @@ from yggdrasil.drivers.InterpretedModelDriver import InterpretedModelDriver
 from yggdrasil.tools import TimeOut, sleep
 logger = logging.getLogger(__name__)
 try:  # pragma: matlab
-    disable_engine = ygg_cfg.get('matlab', 'disable_engine', 'False').lower()
-    if platform._is_win or (disable_engine == 'true'):
+    disable_engine = ygg_cfg.get('matlab', 'disable_engine', 'False')
+    if isinstance(disable_engine, str):
+        disable_engine = (disable_engine.lower() == 'true')
+    if platform._is_win or disable_engine:
         _matlab_engine_installed = False
         if not tools.is_subprocess():
             logger.debug("matlab.engine disabled")
@@ -763,7 +766,7 @@ class MatlabModelDriver(InterpretedModelDriver):  # pragma: matlab
         return super(MatlabModelDriver, cls).executable_command(args, **kwargs)
     
     @classmethod
-    def configure(cls, cfg, disable_engine=None):
+    def configure(cls, cfg, disable_engine=None, hide_matlab_libiomp=None):
         r"""Add configuration options for this language. This includes locating
         any required external libraries and setting option defaults.
 
@@ -772,6 +775,10 @@ class MatlabModelDriver(InterpretedModelDriver):  # pragma: matlab
             disable_engine (bool, optional): Disable the use of the
                 Python engine for Matlab. If not provided, this
                 argument will be ignored.
+            hide_matlab_libiomp (bool, optional): Hide the Matlab version of
+                libiomp by moving it and recording the name of the file. If False,
+                the file will be restored if it exists. If not provided, this
+                argument will be ignored.
 
         Returns:
             list: Section, option, description tuples for options that could not
@@ -800,6 +807,37 @@ class MatlabModelDriver(InterpretedModelDriver):  # pragma: matlab
                     out.append((cls.language, k, opts[k][0]))
         if disable_engine is not None:
             cfg.set(cls._language, 'disable_engine', str(disable_engine))
+        if hide_matlab_libiomp is not None:
+            matlab_root = cfg.get('matlab', 'matlabroot', None)
+            if matlab_root is None:
+                logger.warning("matlabroot not set so we cannot locate the "
+                               "matlab version of libiomp to hide/restore it.")
+                out.append((cls.language, 'hidden_matlab_libiomp',
+                            'The hidden copy of libiomp installed by Matlab'))
+            else:
+                if hide_matlab_libiomp:
+                    if not cfg.get(cls._language, 'hidden_matlab_libiomp', None):
+                        origlib = glob.glob(os.path.join(matlab_root, 'sys', 'os', '*',
+                                                         'libiomp*'))
+                        if len(origlib) == 1:
+                            origlib = origlib[0]
+                            copylib = os.path.join(os.path.dirname(origlib),
+                                                   '_' + os.path.basename(origlib))
+                            cfg.set(cls._language, 'hidden_matlab_libiomp', copylib)
+                            shutil.move(origlib, copylib)
+                        elif len(origlib) == 0:  # pragma: debug
+                            logger.warning("Could not locate the version of libiomp "
+                                           "installed by matlab.")
+                        else:
+                            logger.warning("More than one version of libiomp located "
+                                           "in the Matlab installation: %s" % origlib)
+                else:
+                    copylib = cfg.get(cls._language, 'hidden_matlab_libiomp', None)
+                    if copylib:
+                        origlib = os.path.join(os.path.dirname(copylib),
+                                               os.path.basename(copylib)[1:])
+                        cfg.set(cls._language, 'hidden_matlab_libiomp', '')
+                        shutil.move(copylib, origlib)
         return out
 
     @classmethod
@@ -986,8 +1024,9 @@ class MatlabModelDriver(InterpretedModelDriver):  # pragma: matlab
 
         """
         if comm.direction == 'recv':
-            while comm.recv(timeout=0)[0]:
-                comm.sleep()
+            if not (comm.is_server or comm.is_client):
+                while comm.recv(timeout=0)[0]:
+                    comm.sleep()
         else:
             comm.send_eof()
         comm.linger_close()
