@@ -50,17 +50,14 @@ int init_server_comm(comm_t *comm) {
   } else {
     handle = init_comm_base(comm->name, "recv", _default_comm, dtype_in);
   }
+  handle->flags = handle->flags | COMM_FLAG_SERVER;
   ret = init_default_comm(handle);
   strcpy(comm->address, handle->address);
   // printf("init_server_comm: name = %s, type=%d, address = %s\n",
   // 	 handle->name, handle->type, handle->address);
   strcpy(comm->direction, "recv");
   comm->handle = (void*)handle;
-  if (_default_comm == ZMQ_COMM) {
-    comm->always_send_header = 1;
-  } else {
-    comm->always_send_header = 1; // Always send header.
-  }
+  comm->flags = comm->flags | COMM_ALWAYS_SEND_HEADER;
   comm_t **info = (comm_t**)malloc(sizeof(comm_t*));
   if (info == NULL) {
     ygglog_error("init_server_comm: Failed to malloc info.");
@@ -163,18 +160,18 @@ int server_comm_recv(comm_t* x, char **data, const size_t len, const int allow_r
   }
   // Return EOF
   if (is_eof(*data)) {
-    req_comm->recv_eof[0] = 1;
+    req_comm->const_flags[0] = req_comm->const_flags[0] | COMM_EOF_RECV;
     return ret;
   }
   // Initialize new comm from received address
   comm_head_t head = parse_comm_header(*data, ret);
-  if (!(head.valid)) {
+  if (!(head.flags & HEAD_FLAG_VALID)) {
     ygglog_error("server_comm_recv(%s): Error parsing header.", x->name);
     return -1;
   }
   // Return EOF
   if (is_eof((*data) + head.bodybeg)) {
-    req_comm->recv_eof[0] = 1;
+    req_comm->const_flags[0] = req_comm->const_flags[0] | COMM_EOF_RECV;
     return ret;
   }
   // On client sign off, do a second recv
@@ -202,8 +199,7 @@ int server_comm_recv(comm_t* x, char **data, const size_t len, const int allow_r
     ygglog_error("server_comm_recv(%s): Could not initialize response comm.", x->name);
     return newret;
   }
-  res_comm[0]->sent_eof[0] = 1;
-  res_comm[0]->recv_eof[0] = 1;
+  res_comm[0]->const_flags[0] = res_comm[0]->const_flags[0] | COMM_EOF_SENT | COMM_EOF_RECV;
   return ret;
 };
 
