@@ -21,14 +21,78 @@ extern "C" {
 static void **vcomms2clean = NULL;
 static size_t ncomms2clean = 0;
 static size_t clean_registered = 0;
+static size_t clean_in_progress = 0;
 static size_t clean_called = 0;
+#ifdef _OPENMP
+#pragma omp threadprivate(clean_in_progress)
+#endif
 
 /*! @brief Memory to keep track of global scope comms. */
-static size_t global_scope_comm = 0;
 #ifdef _OPENMP
+static size_t global_scope_comm = 1;
+#define WITH_GLOBAL_SCOPE(COMM) global_scope_comm = 1; COMM
 #pragma omp threadprivate(global_scope_comm)
-#endif
+#else
+static size_t global_scope_comm = 0;
 #define WITH_GLOBAL_SCOPE(COMM) global_scope_comm = 1; COMM; global_scope_comm = 0
+#endif
+
+
+/*!
+ @brief Check if EOF should be sent for a comm being used on multiple threads.
+ @param[in] x const comm_t* Comm to check.
+ @returns int 1 if EOF has been sent for all but this comm and 0 otherwise.
+ */
+static
+int check_threaded_eof(const comm_t* x) {
+  int out = 1;
+#ifdef _OPENMP
+#pragma omp critical (comms)
+  {
+    size_t i;
+    comm_t* icomm = NULL;
+    int nthreads = 1;
+    for (i = 0; i < ncomms2clean; i++) {
+      if ((out == 1) && (vcomms2clean[i] != NULL)) {
+	icomm = (comm_t*)(vcomms2clean[i]);
+	if ((strcmp(icomm->name, x->name) == 0)
+	    && (icomm->thread_id != x->thread_id)) {
+	  nthreads++;
+#pragma omp critical (sent_eof)
+	  {
+	    if ((x->const_flags != NULL) && (!(x->const_flags[0] & COMM_EOF_SENT)))
+	      out = 0;
+	  }
+	}
+      }
+    }
+    if (nthreads < omp_get_num_threads())
+      out = 0;  // all threads havn't initialized a comm
+  }
+#endif
+  return out;
+};
+
+/*!
+  @brief Set the sent_eof flag on the comm.
+  @param[in] x comm_t* Comm to set the flag for.
+*/
+static
+void set_sent_eof(const comm_t* x) {
+#ifdef _OPENMP
+#pragma omp critical (sent_eof)
+  {
+#endif
+  x->const_flags[0] = x->const_flags[0] | COMM_EOF_SENT;
+  if (x->type == CLIENT_COMM) {
+    comm_t *req_comm = (comm_t*)(x->handle);
+    // Don't recurse to prevent block w/ omp critical recursion
+    req_comm->const_flags[0] = req_comm->const_flags[0] | COMM_EOF_SENT;
+  }
+#ifdef _OPENMP
+  }
+#endif
+};
 
 
 /*!
@@ -48,7 +112,6 @@ comm_t* get_global_scope_comm(const char *name) {
     size_t i;
     comm_t* icomm = NULL;
     int current_thread = get_thread_id();
-    printf("CURRENT THREAD: %d\n", current_thread);
     for (i = 0; i < ncomms2clean; i++) {
       if (vcomms2clean[i] != NULL) {
 	icomm = (comm_t*)(vcomms2clean[i]);
@@ -92,6 +155,22 @@ int is_comm_format_array_type(const comm_t *x) {
   return is_dtype_format_array(datatype);
 };
 
+
+/*!
+  @brief Determine if the current thread can use a comm registered by another.
+  @param[in] int Thread that created the comm.
+  @returns int 1 if the current thread can use the comm, 0 otherwise.
+ */
+static
+int thread_can_use(int thread_id) {
+  int current_thread_id = get_thread_id();
+  if ((clean_in_progress) && (current_thread_id == 0))
+    return 1;
+  if (thread_id == current_thread_id)
+    return 1;
+  return 0;
+};
+
   
 /*!
   @brief Perform deallocation for type specific communicator.
@@ -102,7 +181,7 @@ static
 int free_comm_type(comm_t *x) {
   comm_type t = x->type;
   int ret = 1;
-  if (x->thread_id != get_thread_id()) {
+  if (!(thread_can_use(x->thread_id))) {
     ygglog_error("free_comm_type: Thread is attempting to use a comm it did not initialize");
     return ret;
   }
@@ -134,13 +213,9 @@ int free_comm(comm_t *x) {
   int ret = 0;
   if (x == NULL)
     return ret;
-#ifdef _OPENMP
-#pragma omp critical (comms)
-  {
-#endif
   ygglog_debug("free_comm(%s)", x->name);
   // Send EOF for output comms and then wait for messages to be recv'd
-  if ((is_send(x->direction)) && (x->valid)) {
+  if ((is_send(x->direction)) && (x->flags & COMM_FLAG_VALID)) {
     if (_ygg_error_flag == 0) {
       ygglog_debug("free_comm(%s): Sending EOF", x->name);
       comm_send_eof(x);
@@ -153,6 +228,10 @@ int free_comm(comm_t *x) {
       ygglog_error("free_comm(%s): Error registered", x->name);
     }
   }
+#ifdef _OPENMP
+#pragma omp critical (comms)
+  {
+#endif
   ret = free_comm_type(x);
   int idx = x->index_in_register;
   free_comm_base(x);
@@ -174,23 +253,31 @@ int free_comm(comm_t *x) {
 */
 static
 void clean_comms(void) {
-  size_t i;
 #ifdef _OPENMP
-#pragma omp critical (comms)
-  {
+#pragma omp critical (clean)
+    {
 #endif
+  size_t i;
   if (!(clean_called)) {
+    clean_in_progress = 1;
     ygglog_debug("atexit begin");
-    for (i = 0; i < ncomms2clean; i++) {
-      if (vcomms2clean[i] != NULL) {
-	free_comm((comm_t*)(vcomms2clean[i]));
+    if (vcomms2clean != NULL) {
+      for (i = 0; i < ncomms2clean; i++) {
+	if (vcomms2clean[i] != NULL) {
+	  free_comm((comm_t*)(vcomms2clean[i]));
+	}
       }
     }
+#ifdef _OPENMP
+#pragma omp critical (comms)
+    {
+#endif
     if (vcomms2clean != NULL) {
       free(vcomms2clean);
       vcomms2clean = NULL;
     }
     ncomms2clean = 0;
+    ygglog_debug("atexit finished cleaning comms, in final shutdown");
 #if defined(ZMQINSTALLED)
     // #if defined(_MSC_VER) && defined(ZMQINSTALLED)
     ygg_zsys_shutdown();
@@ -200,6 +287,9 @@ void clean_comms(void) {
     }
   /* printf(""); */
     clean_called = 1;
+#ifdef _OPENMP
+    }
+#endif
   }
 #ifdef _OPENMP
   }
@@ -363,7 +453,7 @@ comm_t* new_comm(char *address, const char *direction,
   }
   if (flag < 0) {
     ygglog_error("new_comm: Failed to initialize new comm address.");
-    ret->valid = 0;
+    ret->flags = ret->flags & ~COMM_FLAG_VALID;
   } else {
     if (strlen(ret->name) == 0) {
       sprintf(ret->name, "temp.%s", ret->address);
@@ -371,7 +461,7 @@ comm_t* new_comm(char *address, const char *direction,
     flag = register_comm(ret);
     if (flag < 0) {
       ygglog_error("new_comm: Failed to register new comm.");
-      ret->valid = 0;
+      ret->flags = ret->flags & ~COMM_FLAG_VALID;
     }
   }
   return ret;
@@ -399,6 +489,7 @@ comm_t* init_comm(const char *name, const char *direction,
 #endif
   comm_t *ret = get_global_scope_comm(name);
   if (ret != NULL) {
+    destroy_dtype(&datatype);
     return ret;
   }
   if ((datatype == NULL) && (strcmp(direction, "send") == 0)) {
@@ -412,19 +503,21 @@ comm_t* init_comm(const char *name, const char *direction,
   int flag = init_comm_type(ret);
   if (flag < 0) {
     ygglog_error("init_comm(%s): Could not initialize comm.", name);
-    ret->valid = 0;
+    ret->flags = ret->flags & ~COMM_FLAG_VALID;
   } else {
     flag = register_comm(ret);
     if (flag < 0) {
       ygglog_error("init_comm(%s): Failed to register new comm.", name);
-      ret->valid = 0;
+      ret->flags = ret->flags & ~COMM_FLAG_VALID;
     }
   }
-  if (global_scope_comm) {
-    ret->is_global = 1;
-    ygglog_debug("init_comm(%s): Global comm!", name);
+  if (ret->flags & COMM_FLAG_VALID) {
+    if (global_scope_comm) {
+      ret->flags = ret->flags | COMM_FLAG_GLOBAL;
+      ygglog_debug("init_comm(%s): Global comm!", name);
+    }
+    ygglog_debug("init_comm(%s): Initialized comm.", name);
   }
-  ygglog_debug("init_comm(%s): Initialized comm.", name);
   return ret;
 };
 
@@ -467,7 +560,7 @@ comm_t* init_comm_format(const char *name, const char *direction,
   if ((format_str != NULL) && (datatype == NULL)) {
     ygglog_error("init_comm_format: Failed to create type from format_str.");
     if (out != NULL) {
-      out->valid = 0;
+      out->flags = out->flags & ~COMM_FLAG_VALID;
     }
   }
   return out;
@@ -482,7 +575,7 @@ comm_t* init_comm_format(const char *name, const char *direction,
 static
 int comm_nmsg(const comm_t *x) {
   int ret = -1;
-  if ((x == NULL) || (x->valid == 0)) {
+  if ((x == NULL) || (!(x->flags & COMM_FLAG_VALID))) {
     ygglog_error("comm_nmsg: Invalid comm");
     return ret;
   }
@@ -518,11 +611,11 @@ static
 int comm_send_single(const comm_t *x, const char *data, const size_t len) {
   ygglog_debug("Sending %d bytes: '%s'\n", len, data);
   int ret = -1;
-  if ((x == NULL) || (x->valid == 0)) {
+  if ((x == NULL) || (!(x->flags & COMM_FLAG_VALID))) {
     ygglog_error("comm_send_single: Invalid comm");
     return ret;
   }
-  if (x->thread_id != get_thread_id()) {
+  if (!(thread_can_use(x->thread_id))) {
     ygglog_error("comm_send_single: Thread is attempting to use a comm it did not initialize");
     return ret;
   }
@@ -565,10 +658,13 @@ comm_head_t comm_send_multipart_header(const comm_t *x, const char * data,
 				       const size_t len) {
   comm_head_t head = init_header(len, NULL, NULL);
   sprintf(head.id, "%d", rand());
-  head.multipart = 1;
-  head.valid = 1;
+  char *model_name = getenv("YGG_MODEL_NAME");
+  if (model_name != NULL) {
+    strcpy(head.model, model_name);
+  }
+  head.flags = head.flags | HEAD_FLAG_VALID | HEAD_FLAG_MULTIPART;
   // Add datatype information to header
-  if (x->is_file == 0) {
+  if (!(x->flags & COMM_FLAG_FILE)) {
     dtype_t *datatype;
     if (x->type == CLIENT_COMM) {
       comm_t *req_comm = (comm_t*)(x->handle);
@@ -584,7 +680,7 @@ comm_head_t comm_send_multipart_header(const comm_t *x, const char * data,
     if (res_comm[0] == NULL) {
       ygglog_error("comm_send_multipart_header(%s): no response comm registered",
 		   x->name);
-      head.valid = 0;
+      head.flags = head.flags & ~HEAD_FLAG_VALID;
       return head;
     }
     x0 = &((*res_comm)[0]);
@@ -603,7 +699,7 @@ comm_head_t comm_send_multipart_header(const comm_t *x, const char * data,
     char *reply_address = set_reply_send(x0);
     if (reply_address == NULL) {
       ygglog_error("comm_send_multipart_header: Could not set reply address.");
-      head.valid = 0;
+      head.flags = head.flags & ~HEAD_FLAG_VALID;
       return head;
     }
     strcpy(head.zmq_reply, reply_address);
@@ -626,13 +722,13 @@ int comm_send_multipart(const comm_t *x, const char *data, const size_t len) {
   int headlen = 0, ret = -1;
   comm_t* xmulti = NULL;
   int no_type = is_eof(data);
-  if ((x == NULL) || (x->valid == 0)) {
+  if ((x == NULL) || (!(x->flags & COMM_FLAG_VALID))) {
     ygglog_error("comm_send_multipart: Invalid comm");
     return ret;
   }
   // Get header
   comm_head_t head = comm_send_multipart_header(x, data, len);
-  if (head.valid == 0) {
+  if (!(head.flags & HEAD_FLAG_VALID)) {
     ygglog_error("comm_send_multipart: Invalid header generated.");
     return -1;
   }
@@ -662,24 +758,23 @@ int comm_send_multipart(const comm_t *x, const char *data, const size_t len) {
 	headbuf = t_headbuf;
         headbuf_len = (size_t)headlen + len + 1;
       }
-      head.multipart = 0;
+      head.flags = head.flags & ~HEAD_FLAG_MULTIPART;
       memcpy(headbuf + headlen, data, len);
       headlen += (int)len;
       headbuf[headlen] = '\0';
     }
   }
   // Get head string
-  if (head.multipart == 1) {
+  if (head.flags & HEAD_FLAG_MULTIPART) {
     // Get address for new comm and add to header
     xmulti = new_comm(NULL, "send", x->type, NULL);
-    if ((xmulti == NULL) || (!(xmulti->valid))) {
+    if ((xmulti == NULL) || (!(xmulti->flags & COMM_FLAG_VALID))) {
       ygglog_error("comm_send_multipart: Failed to initialize a new comm.");
       free(headbuf);
       return -1;
     }
-    xmulti->sent_eof[0] = 1;
-    xmulti->recv_eof[0] = 1;
-    xmulti->is_work_comm = 1;
+    xmulti->const_flags[0] = xmulti->const_flags[0] | COMM_EOF_SENT | COMM_EOF_RECV;
+    xmulti->flags = xmulti->flags | COMM_FLAG_WORKER;
     strcpy(head.address, xmulti->address);
     if (xmulti->type == ZMQ_COMM) {
       char *reply_address = set_reply_send(xmulti);
@@ -705,7 +800,7 @@ int comm_send_multipart(const comm_t *x, const char *data, const size_t len) {
   }
   // Send header
   size_t data_in_header = 0;
-  if ((head.type_in_data) && (headlen > (x->maxMsgSize - x->msgBufSize))) {
+  if ((head.flags & HEAD_TYPE_IN_DATA) && ((size_t)headlen > (x->maxMsgSize - x->msgBufSize))) {
     ret = comm_send_single(x, headbuf, x->maxMsgSize - x->msgBufSize);
     data_in_header = headlen - (x->maxMsgSize - x->msgBufSize);
   } else {
@@ -719,7 +814,7 @@ int comm_send_multipart(const comm_t *x, const char *data, const size_t len) {
     free(headbuf);
     return -1;
   }
-  if (head.multipart == 0) {
+  if (!(head.flags & HEAD_FLAG_MULTIPART)) {
     ygglog_debug("comm_send_multipart(%s): %d bytes completed", x->name, head.size);
     free(headbuf);
     return ret;
@@ -727,7 +822,7 @@ int comm_send_multipart(const comm_t *x, const char *data, const size_t len) {
   // Send data stored in header
   size_t msgsiz;
   size_t prev = headlen - data_in_header;
-  while (prev < headlen) {
+  while (prev < (size_t)headlen) {
     if ((headlen - prev) > (xmulti->maxMsgSize - xmulti->msgBufSize)) {
       msgsiz = xmulti->maxMsgSize - xmulti->msgBufSize;
     } else {
@@ -778,7 +873,7 @@ int comm_send_multipart(const comm_t *x, const char *data, const size_t len) {
   }
   free(headbuf);
   if (ret >= 0)
-    x->used[0] = 1;
+    x->const_flags[0] = x->const_flags[0] | COMM_FLAGS_USED;
   return ret;
 };
 
@@ -795,31 +890,32 @@ int comm_send_multipart(const comm_t *x, const char *data, const size_t len) {
 static
 int comm_send(const comm_t *x, const char *data, const size_t len) {
   int ret = -1;
-  if ((x == NULL) || (x->valid == 0)) {
+  if ((x == NULL) || (!(x->flags & COMM_FLAG_VALID))) {
     ygglog_error("comm_send: Invalid comm");
     return ret;
   }
-  if (x->sent_eof == NULL) {
-    ygglog_error("comm_send(%s): sent_eof not initialized.", x->name);
+  if (x->const_flags == NULL) {
+    ygglog_error("comm_send(%s): const_flags not initialized.", x->name);
     return ret;
   }
   int sending_eof = 0;
   if (is_eof(data)) {
-    if (x->sent_eof[0] == 0) {
-      x->sent_eof[0] = 1;
-      if (x->type == CLIENT_COMM) {
-	comm_t *req_comm = (comm_t*)(x->handle);
-	req_comm->sent_eof[0] = 1;
-      }
-      sending_eof = 1;
-      ygglog_debug("comm_send(%s): Sending EOF", x->name);
-    } else {
+    if (x->const_flags[0] & COMM_EOF_SENT) {
       ygglog_debug("comm_send(%s): EOF already sent", x->name);
       return ret;
+    } else if (!(check_threaded_eof(x))) {
+      ygglog_debug("comm_send(%s): EOF not sent on other threads", x->name);
+      set_sent_eof(x);
+      return 0;
+    } else {
+      set_sent_eof(x);
+      sending_eof = 1;
+      ygglog_debug("comm_send(%s): Sending EOF", x->name);
     }
   }
   if (((len > x->maxMsgSize) && (x->maxMsgSize > 0)) ||
-      (((x->always_send_header) || (x->used[0] == 0)))) {
+      (((x->flags & COMM_ALWAYS_SEND_HEADER) ||
+	(!(x->const_flags[0] & COMM_FLAGS_USED))))) {
     ygglog_debug("comm_send(%s): Sending as one or more messages with a header.",
 		 x->name);
     ret = comm_send_multipart(x, data, len);
@@ -832,7 +928,7 @@ int comm_send(const comm_t *x, const char *data, const size_t len) {
     ygglog_debug("comm_send(%s): sent EOF, ret = %d", x->name, ret);
   }
   if (ret >= 0)
-    x->used[0] = 1;
+    x->const_flags[0] = x->const_flags[0] | COMM_FLAGS_USED;
   return ret;
 };
 
@@ -865,11 +961,11 @@ static
 int comm_recv_single(comm_t *x, char **data, const size_t len,
 		     const int allow_realloc) {
   int ret = -1;
-  if ((x == NULL) || (x->valid == 0)) {
+  if ((x == NULL) || (!(x->flags & COMM_FLAG_VALID))) {
     ygglog_error("comm_recv_single: Invalid comm");
     return ret;
   }
-  if (x->thread_id != get_thread_id()) {
+  if (!(thread_can_use(x->thread_id))) {
     ygglog_error("comm_recv_single: Thread is attempting to use a comm it did not initialize");
     return ret;
   }
@@ -906,13 +1002,13 @@ static
 int comm_recv_multipart(comm_t *x, char **data, const size_t len,
 			const size_t headlen, const int allow_realloc) {
   int ret = -1;
-  if ((x == NULL) || (x->valid == 0)) {
+  if ((x == NULL) || (!(x->flags & COMM_FLAG_VALID))) {
     ygglog_error("comm_recv_multipart: Invalid comm");
     return ret;
   }
   usleep(100);
   comm_head_t head = parse_comm_header(*data, headlen);
-  if (!(head.valid)) {
+  if (!(head.flags & HEAD_FLAG_VALID)) {
     ygglog_error("comm_recv_multipart(%s): Error parsing header.", x->name);
     ret = -1;
   } else {
@@ -921,7 +1017,8 @@ int comm_recv_multipart(comm_t *x, char **data, const size_t len,
     (*data)[head.bodysiz] = '\0';
     if (is_eof(*data)) {
       ygglog_debug("comm_recv_multipart(%s): EOF received.", x->name);
-      x->recv_eof[0] = 1;
+      x->const_flags[0] = x->const_flags[0] | COMM_EOF_RECV;
+      destroy_header(&head);
       return -2;
     }
     // Get datatype information from header on first recv
@@ -932,40 +1029,44 @@ int comm_recv_multipart(comm_t *x, char **data, const size_t len,
     } else {
       updtype = x->datatype;
     }
-    if ((x->used[0] == 0) && (x->is_file == 0) && (updtype->obj == NULL) && (head.type_in_data == 0)) {
+    if ((!(x->const_flags[0] & COMM_FLAGS_USED)) && (!(x->flags & COMM_FLAG_FILE)) && (updtype->obj == NULL) && (!(head.flags & HEAD_TYPE_IN_DATA))) {
       ygglog_debug("comm_recv_multipart(%s): Updating datatype to '%s'",
 		   x->name, head.dtype->type);
       ret = update_dtype(updtype, head.dtype);
       if (ret != 0) {
 	ygglog_error("comm_recv_multipart(%s): Error updating datatype.", x->name);
+	destroy_header(&head);
 	return -1;
       }
-    } else if ((x->is_file == 0) && (head.dtype != NULL)) {
+    } else if ((!(x->flags & COMM_FLAG_FILE)) && (head.dtype != NULL)) {
       ret = update_dtype(updtype, head.dtype);
       if (ret != 0) {
 	ygglog_error("comm_recv_multipart(%s): Error updating existing datatype.", x->name);
+	destroy_header(&head);
 	return -1;
       }
     }
-    if (head.multipart) {
+    if (head.flags & HEAD_FLAG_MULTIPART) {
       // Return early if header contained entire message
       if (head.size == head.bodysiz) {
-        x->used[0] = 1;
+        x->const_flags[0] = x->const_flags[0] | COMM_FLAGS_USED;
+	destroy_header(&head);
 	return (int)(head.bodysiz);
       }
       // Get address for new comm
       comm_t* xmulti = new_comm(head.address, "recv", x->type, NULL);
-      if ((xmulti == NULL) || (!(xmulti->valid))) {
+      if ((xmulti == NULL) || (!(xmulti->flags & COMM_FLAG_VALID))) {
 	ygglog_error("comm_recv_multipart: Failed to initialize a new comm.");
+	destroy_header(&head);
 	return -1;
       }
-      xmulti->sent_eof[0] = 1;
-      xmulti->recv_eof[0] = 1;
-      xmulti->is_work_comm = 1;
+      xmulti->const_flags[0] = xmulti->const_flags[0] | COMM_EOF_SENT | COMM_EOF_RECV;
+      xmulti->flags = xmulti->flags | COMM_FLAG_WORKER;
       if (xmulti->type == ZMQ_COMM) {
 	int reply_socket = set_reply_recv(xmulti, head.zmq_reply_worker);
 	if (reply_socket < 0) {
 	  ygglog_error("comm_recv_multipart: Failed to set worker reply address.");
+	  destroy_header(&head);
 	  return -1;
 	}
       }
@@ -981,6 +1082,7 @@ int comm_recv_multipart(comm_t *x, char **data, const size_t len,
 			 x->name);
 	    free(*data);
 	    free_comm(xmulti);
+	    destroy_header(&head);
 	    return -1;
 	  }
 	  *data = t_data;
@@ -988,6 +1090,7 @@ int comm_recv_multipart(comm_t *x, char **data, const size_t len,
 	  ygglog_error("comm_recv_multipart(%s): buffer is not large enough",
 		       x->name);
 	  free_comm(xmulti);
+	  destroy_header(&head);
 	  return -1;
 	}
       }
@@ -1006,7 +1109,7 @@ int comm_recv_multipart(comm_t *x, char **data, const size_t len,
 	ygglog_debug("comm_recv_multipart(%s): %d of %d bytes received",
 		     x->name, prev, head.size);
       }
-      if ((ret > 0) && (head.type_in_data)) {
+      if ((ret > 0) && (head.flags & HEAD_TYPE_IN_DATA)) {
 	ygglog_debug("comm_recv_multipart(%s): Extracting type from data.");
 	ret = parse_type_in_data(data, prev, &head);
 	if (ret > 0) {
@@ -1014,6 +1117,7 @@ int comm_recv_multipart(comm_t *x, char **data, const size_t len,
 	  ret = update_dtype(updtype, head.dtype);
 	  if (ret != 0) {
 	    ygglog_error("comm_recv_multipart(%s): Error updating existing datatype.", x->name);
+	    destroy_header(&head);
 	    return -1;
 	  } else {
 	    ret = (int)prev;
@@ -1030,7 +1134,8 @@ int comm_recv_multipart(comm_t *x, char **data, const size_t len,
     }
   }
   if (ret >= 0)
-    x->used[0] = 1;
+    x->const_flags[0] = x->const_flags[0] | COMM_FLAGS_USED;
+  destroy_header(&head);
   return ret;
 };
 
@@ -1050,7 +1155,7 @@ int comm_recv(comm_t *x, char *data, const size_t len) {
   if (ret > 0) {
     if (is_eof(data)) {
       ygglog_debug("comm_recv(%s): EOF received.", x->name);
-      x->recv_eof[0] = 1;
+      x->const_flags[0] = x->const_flags[0] | COMM_EOF_RECV;
       ret = -2;
     } else {
       ret = comm_recv_multipart(x, &data, len, ret, 0);
@@ -1077,7 +1182,7 @@ int comm_recv_realloc(comm_t *x, char **data, const size_t len) {
   if (ret > 0) {
     if (is_eof(*data)) {
       ygglog_debug("comm_recv_realloc(%s): EOF received.", x->name);
-      x->recv_eof[0] = 1;
+      x->const_flags[0] = x->const_flags[0] | COMM_EOF_RECV;
       ret = -2;
     } else {
       ret = comm_recv_multipart(x, data, len, ret, 1);
@@ -1104,18 +1209,18 @@ int comm_send_nolimit(const comm_t *x, const char *data, const size_t len) {
 static
 int comm_send_nolimit_eof(const comm_t *x) {
   int ret = -1;
-  if ((x == NULL) || (x->valid == 0)) {
+  if ((x == NULL) || (!(x->flags & COMM_FLAG_VALID))) {
     ygglog_error("comm_send_nolimit_eof: Invalid comm");
     return ret;
   }
-  if (x->sent_eof == NULL) {
-    ygglog_error("comm_send_nolimit_eof(%s): sent_eof not initialized.", x->name);
+  if (x->const_flags == NULL) {
+    ygglog_error("comm_send_nolimit_eof(%s): const_flags not initialized.", x->name);
     return ret;
   }
-  if (x->sent_eof[0] == 0) {
+  if (!(x->const_flags[0] & COMM_EOF_SENT)) {
     char buf[2048] = YGG_MSG_EOF;
     ret = comm_send_nolimit(x, buf, strlen(buf));
-    x->sent_eof[0] = 1;
+    set_sent_eof(x);
   } else {
     ygglog_debug("comm_send_nolimit_eof(%s): EOF already sent", x->name);
   }
@@ -1155,7 +1260,7 @@ static
 int vcommSend(const comm_t *x, size_t nargs, va_list_t ap) {
   ygglog_debug("vcommSend: Formatting %lu arguments.", nargs);
   int ret = -1;
-  if ((x == NULL) || (x->valid == 0)) {
+  if ((x == NULL) || (!(x->flags & COMM_FLAG_VALID))) {
     ygglog_error("vcommSend: Invalid comm");
     return ret;
   }
@@ -1235,7 +1340,7 @@ static
 int vcommRecv(comm_t *x, const int allow_realloc, size_t nargs, va_list_t ap) {
   int ret = -1;
   ygglog_debug("vcommRecv: Parsing %lu arguments.", nargs);
-  if ((x == NULL) || (x->valid == 0)) {
+  if ((x == NULL) || (!(x->flags & COMM_FLAG_VALID))) {
     ygglog_error("vcommRecv: Invalid comm");
     return ret;
   }
